@@ -54,7 +54,7 @@ namespace pappso
 QMutex AaModification::m_mutex;
 
 AaModification::AaModification(const QString &accession, pappso_double mass)
-  : m_mass(mass), m_accession(accession)
+  : m_accession(accession), m_mass(mass)
 {
   m_atomCount = {{AtomIsotopeSurvey::C, 0},
                  {AtomIsotopeSurvey::H, 0},
@@ -74,10 +74,11 @@ AaModification::AaModification(const QString &accession, pappso_double mass)
 
 
 AaModification::AaModification(AaModification &&toCopy) // move constructor
-  : m_mass(toCopy.m_mass),
+  : m_accession(toCopy.m_accession),
+    m_name(toCopy.m_name),
+    m_mass(toCopy.m_mass),
     m_atomCount(std::move(toCopy.m_atomCount)),
-    m_mapIsotope(toCopy.m_mapIsotope),
-    m_accession(toCopy.m_accession)
+    m_mapIsotope(toCopy.m_mapIsotope)
 {
   m_origin = toCopy.m_origin;
 }
