@@ -212,6 +212,7 @@ MsRunRetentionTime<T>::computePeptideRetentionTimes()
       throw ExceptionNotPossible(
         QObject::tr("ERROR : msp_msrunReader == nullptr"));
     }
+  msp_msrunReader.get()->releaseDevice();
   m_seamarks.clear();
   if(m_retentionTimeReferenceMethod ==
      ComputeRetentionTimeReference::maximum_intensity)
@@ -243,7 +244,6 @@ MsRunRetentionTime<T>::computePeptideRetentionTimes()
           it++;
         }
     }
-  msp_msrunReader = nullptr;
   m_allMs2Points.clear();
 
   std::sort(m_seamarks.begin(),
