@@ -787,6 +787,14 @@ void
 PwizMsRunReader::readSpectrumCollection(
   SpectrumCollectionHandlerInterface &handler)
 {
+  readSpectrumCollectionByMsLevel(handler, 0);
+}
+
+
+void
+PwizMsRunReader::readSpectrumCollectionByMsLevel(
+  SpectrumCollectionHandlerInterface &handler, unsigned int ms_level)
+{
 
   acquireDevice();
   // qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()";
@@ -896,7 +904,17 @@ PwizMsRunReader::readSpectrumCollection(
 
       // The handler will receive the index of the mass spectrum in the
       // current run via the mass spectrum id member datum.
-      handler.setQualifiedMassSpectrum(qualified_mass_spectrum);
+      if(ms_level == 0)
+        {
+          handler.setQualifiedMassSpectrum(qualified_mass_spectrum);
+        }
+      else
+        {
+          if(qualified_mass_spectrum.getMsLevel() == ms_level)
+            {
+              handler.setQualifiedMassSpectrum(qualified_mass_spectrum);
+            }
+        }
     }
 
   setlocale(LC_ALL, env.c_str());
@@ -911,7 +929,6 @@ PwizMsRunReader::readSpectrumCollection(
   handler.loadingEnded();
 }
 
-
 std::size_t
 PwizMsRunReader::spectrumListSize() const
 {
