@@ -57,6 +57,11 @@ class TimsMsRunReader : public MsRunReader
   virtual void
   readSpectrumCollection(SpectrumCollectionHandlerInterface &handler) override;
 
+  virtual void
+  readSpectrumCollectionByMsLevel(SpectrumCollectionHandlerInterface &handler,
+                                  unsigned int ms_level) override;
+
+
   virtual std::size_t spectrumListSize() const override;
 
   virtual bool hasScanNumbers() const override;
