@@ -52,6 +52,13 @@ void
 pappso::TimsMsRunReaderMs2::initialize()
 {
   msp_timsData = std::make_shared<TimsData>(mcsp_msRunId.get()->getFileName());
+  if(msp_timsData == nullptr)
+    {
+      throw PappsoException(
+        QObject::tr("ERROR in TimsMsRunReaderMs2::initialize "
+                    "msp_timsData is null for MsRunId %1")
+          .arg(mcsp_msRunId.get()->toString()));
+    }
 }
 
 void
@@ -171,6 +178,14 @@ void
 TimsMsRunReaderMs2::readSpectrumCollection(
   SpectrumCollectionHandlerInterface &handler)
 {
+  readSpectrumCollectionByMsLevel(handler, 0);
+}
+
+void
+TimsMsRunReaderMs2::readSpectrumCollectionByMsLevel(
+  SpectrumCollectionHandlerInterface &handler, unsigned int ms_level [[maybe_unused]])
+{
+
   const bool want_binary_data = handler.needPeakList();
   // const bool want_binary_data = false;
 
