@@ -27,6 +27,12 @@ class XyMsRunReader : public MsRunReader
   virtual void
   readSpectrumCollection(SpectrumCollectionHandlerInterface &handler) override;
 
+
+  virtual void
+  readSpectrumCollectionByMsLevel(SpectrumCollectionHandlerInterface &handler,
+                                  unsigned int ms_level) override;
+
+
   virtual std::size_t spectrumListSize() const override;
 
   virtual bool releaseDevice() override;
