@@ -154,7 +154,7 @@ class PMSPP_LIB_DECL FilterMorphoMaxMin : public FilterInterface
 /** @brief anti spike filter
  * set to zero alone values inside the window
  */
-class PMSPP_LIB_DECL FilterMorphoAntiSpike : public FilterInterface
+class PMSPP_LIB_DECL FilterMorphoAntiSpike : public FilterNameInterface
 {
   private:
   std::size_t m_halfWindowSize = 0;
@@ -168,6 +168,11 @@ class PMSPP_LIB_DECL FilterMorphoAntiSpike : public FilterInterface
   Trace &filter(Trace &data_points) const override;
 
   std::size_t getHalfWindowSize() const;
+
+
+  void buildFilterFromString(const QString &strBuildParams) override;
+
+  QString toString() const override;
 };
 
 
