@@ -74,6 +74,11 @@ TandemWrapperRun::~TandemWrapperRun()
     {
       delete mpa_temporaryDirectory;
     }
+
+  if(m_xtProcess != nullptr)
+    {
+      m_xtProcess->deleteLater();
+    }
 }
 
 void
@@ -284,8 +289,17 @@ TandemWrapperRun::readTandemPresetFile(const QString &tandem_preset_file)
         }
 
       QString ms2_filters_str = preset_handler.getMs2FiltersOptions();
-      msp_ms2FilterSuiteString =
-        std::make_shared<pappso::FilterSuiteString>(ms2_filters_str);
+      if(!ms2_filters_str.isEmpty())
+        {
+          msp_ms2FilterSuiteString =
+            std::make_shared<pappso::FilterSuiteString>(ms2_filters_str);
+        }
+      else
+        {
+          msp_ms2FilterSuiteString =
+            std::make_shared<pappso::FilterSuiteString>(
+              "chargeDeconvolution|0.02dalton mzExclusion|0.01dalton");
+        }
     }
   else
     {
@@ -314,6 +328,12 @@ TandemWrapperRun::wrapTandemInputFile(const QString &tandem_input_file)
     mzxml_data_file_name, wrapped_tandem_input, wrapped_tandem_output);
 
   QFile qfile(tandem_input_file);
+  if(!qfile.exists())
+    {
+      throw pappso::PappsoException(
+        QObject::tr("Tandem input file %1 does not exists")
+          .arg(QFileInfo(tandem_input_file).absoluteFilePath()));
+    }
   QXmlInputSource xmlInputSource(&qfile);
   QXmlSimpleReader simplereader;
   simplereader.setContentHandler(&wrap_input);
@@ -393,6 +413,8 @@ TandemWrapperRun::convertOrginalMsData2mzXmlData(const QString &origin,
       p_mzxml_output->write(p_reader.get());
 
       p_mzxml_output->close();
+
+      delete p_mzxml_output;
     }
   else
     {
