@@ -119,9 +119,7 @@ class PMSPP_LIB_DECL TandemWrapperRun : public QObject
   int m_maxTandemRunTimeMs = (60000 * 60 * 24); // 1 day
   QProcess *m_xtProcess    = nullptr;
 
-  std::shared_ptr<FilterSuiteString> msp_ms2FilterSuiteString =
-    std::make_shared<pappso::FilterSuiteString>(
-      "chargeDeconvolution|0.02dalton");
+  std::shared_ptr<FilterSuiteString> msp_ms2FilterSuiteString = nullptr;
 
   QTextStream *mp_outputStream = nullptr;
   QTextStream *mp_errorStream  = nullptr;
