@@ -66,6 +66,7 @@ class PMSPP_LIB_DECL Utils
 
   static int zeroDecimalsInValue(pappso_double value);
   static pappso_double roundToDecimals(pappso_double value, int decimal_places);
+  static long long int roundToDecimal32bitsAsLongLongInt(pappso::pappso_double input);
 
   static std::string toUtf8StandardString(const QString &text);
 
@@ -78,6 +79,8 @@ class PMSPP_LIB_DECL Utils
 
   static bool almostEqual(double value1, double value2, int decimalPlaces = 10);
 
+  static double nearestGreater(double value);
+
   static QString
   chronoTimePointDebugString(const QString &msg,
                              std::chrono::system_clock::time_point chrono_time =
