@@ -115,6 +115,8 @@ TimsData::TimsData(QDir timsDataDirectory)
     }
 
 
+  fillFrameIdDescrList();
+
   // get number of scans
   if(!q.exec("SELECT SUM(NumScans) FROM Frames"))
     {
@@ -153,6 +155,54 @@ TimsData::TimsData(QDir timsDataDirectory)
         std::pair<int, QSqlRecord>(record.value(0).toInt(), record));
     }
 
+  // m_mapTimsCalibrationRecord
+
+  if(!q.exec("select * from TimsCalibration;"))
+    {
+      qDebug();
+      throw PappsoException(
+        QObject::tr("ERROR in TIMS sqlite database file %1, executing SQL "
+                    "command %2:\n%3\n%4\n%5")
+          .arg(m_timsDataDirectory.absoluteFilePath("analysis.tdf"))
+          .arg(q.lastQuery())
+          .arg(q.lastError().databaseText())
+          .arg(q.lastError().driverText())
+          .arg(q.lastError().nativeErrorCode()));
+    }
+  while(q.next())
+    {
+      QSqlRecord record = q.record();
+      m_mapTimsCalibrationRecord.insert(
+        std::pair<int, QSqlRecord>(record.value(0).toInt(), record));
+    }
+
+
+  // store frames
+  if(!q.exec("select Frames.TimsId, Frames.AccumulationTime, "        // 1
+             "Frames.MzCalibration, "                                 // 2
+             "Frames.T1, Frames.T2, "                                 // 4
+             "Frames.Time, Frames.MsMsType, Frames.TimsCalibration, " // 7
+             "Frames.Id "                                             // 8
+             " FROM Frames;"))
+    {
+      qDebug();
+      throw PappsoException(
+        QObject::tr("ERROR in TIMS sqlite database file %1, executing SQL "
+                    "command %2:\n%3\n%4\n%5")
+          .arg(m_timsDataDirectory.absoluteFilePath("analysis.tdf"))
+          .arg(q.lastQuery())
+          .arg(q.lastError().databaseText())
+          .arg(q.lastError().driverText())
+          .arg(q.lastError().nativeErrorCode()));
+    }
+  while(q.next())
+    {
+      QSqlRecord record = q.record();
+      m_mapFramesRecord.insert(
+        std::pair<std::size_t, QSqlRecord>(record.value(8).toUInt(), record));
+    }
+
+
   std::shared_ptr<pappso::FilterPseudoCentroid> ms2filter =
     std::make_shared<pappso::FilterPseudoCentroid>(35000, 5, 0.5, 0.1);
   mcsp_ms2Filter = ms2filter;
@@ -207,6 +257,10 @@ TimsData::~TimsData()
     {
       delete mpa_timsBinDec;
     }
+  if(mpa_mzCalibrationStore != nullptr)
+    {
+      delete mpa_mzCalibrationStore;
+    }
 }
 
 void
@@ -221,8 +275,8 @@ TimsData::getMs2BuiltinCentroid() const
   return m_builtinMs2Centroid;
 }
 
-std::pair<std::size_t, std::size_t>
-TimsData::getScanCoordinateFromRawIndex(std::size_t raw_index) const
+void
+TimsData::fillFrameIdDescrList()
 {
 
   QSqlDatabase qdb = openDatabaseConnection();
@@ -246,19 +300,48 @@ TimsData::getScanCoordinateFromRawIndex(std::size_t raw_index) const
   bool index_found = false;
   std::size_t timsId;
   std::size_t numberScans;
-  std::size_t offset = 0;
+  std::size_t cumulScans = 0;
   while(q.next() && (!index_found))
     {
       timsId      = q.value(0).toUInt();
       numberScans = q.value(1).toUInt();
 
-      if(raw_index < (offset + numberScans))
-        {
-          return std::pair<std::size_t, std::size_t>(timsId,
-                                                     raw_index - offset);
-        }
+      m_thousandIndexToFrameIdDescrListIndex.insert(
+        std::pair<std::size_t, std::size_t>((cumulScans % 1000),
+                                            m_frameIdDescrList.size()));
 
-      offset += numberScans;
+      m_frameIdDescrList.push_back({timsId, numberScans, cumulScans});
+      cumulScans += numberScans;
+    }
+}
+
+std::pair<std::size_t, std::size_t>
+TimsData::getScanCoordinateFromRawIndex(std::size_t raw_index) const
+{
+
+  std::size_t modulo = raw_index % 1000;
+  auto map_it        = m_thousandIndexToFrameIdDescrListIndex.find(modulo);
+  if(map_it == m_thousandIndexToFrameIdDescrListIndex.end())
+    {
+      throw ExceptionNotFound(
+        QObject::tr("ERROR raw index %1 not found (modulo)").arg(raw_index));
+    }
+  std::size_t start_point_index = map_it->second;
+  while((start_point_index > 0) &&
+        (m_frameIdDescrList[start_point_index].m_cumulSize > raw_index))
+    {
+      start_point_index--;
+    }
+  for(std::size_t i = start_point_index; i < m_frameIdDescrList.size(); i++)
+    {
+
+      if(raw_index <
+         (m_frameIdDescrList[i].m_cumulSize + m_frameIdDescrList[i].m_size))
+        {
+          return std::pair<std::size_t, std::size_t>(
+            m_frameIdDescrList[i].m_frameId,
+            raw_index - m_frameIdDescrList[i].m_cumulSize);
+        }
     }
 
   throw ExceptionNotFound(
@@ -271,38 +354,12 @@ TimsData::getRawIndexFromCoordinate(std::size_t frame_id,
                                     std::size_t scan_num) const
 {
 
-
-  QSqlDatabase qdb = openDatabaseConnection();
-  QSqlQuery q =
-    qdb.exec(QString("SELECT Id, NumScans FROM "
-                     "Frames ORDER BY Id"));
-  if(q.lastError().isValid())
+  for(auto frameDescr : m_frameIdDescrList)
     {
-
-      throw PappsoException(
-        QObject::tr("ERROR in TIMS sqlite database file %1, executing SQL "
-                    "command %2:\n%3\n%4\n%5")
-          .arg(m_timsDataDirectory.absoluteFilePath("analysis.tdf"))
-          .arg(q.lastQuery())
-          .arg(qdb.lastError().databaseText())
-          .arg(qdb.lastError().driverText())
-          .arg(qdb.lastError().nativeErrorCode()));
-    }
-  bool index_found = false;
-  std::size_t timsId;
-  std::size_t numberScans;
-  std::size_t offset = 0;
-  while(q.next() && (!index_found))
-    {
-      timsId      = q.value(0).toUInt();
-      numberScans = q.value(1).toUInt();
-
-      if(timsId == frame_id)
+      if(frameDescr.m_frameId == frame_id)
         {
-          return offset + scan_num;
+          return frameDescr.m_cumulSize + scan_num;
         }
-
-      offset += numberScans;
     }
 
   throw ExceptionNotFound(
@@ -330,82 +387,73 @@ TimsData::getTimsFrameBaseCstSPtr(std::size_t timsId) const
 
   qDebug() << " timsId=" << timsId;
 
-  QSqlDatabase qdb = openDatabaseConnection();
-  QSqlQuery q      = qdb.exec(
-    QString("SELECT Frames.TimsId, Frames.AccumulationTime, " // 1
-            "Frames.MzCalibration, "                          // 2
-            "Frames.T1, Frames.T2, "                          // 4
-            "Frames.Time, Frames.MsMsType, TimsCalibration.ModelType, " // 7
-            "TimsCalibration.C0, TimsCalibration.C1, TimsCalibration.C2, " // 10
-            "TimsCalibration.C3, TimsCalibration.C4, TimsCalibration.C5, " // 13
-            "TimsCalibration.C6, TimsCalibration.C7, TimsCalibration.C8, " // 16
-            "TimsCalibration.C9   FROM  Frames" // 17
-            " INNER JOIN TimsCalibration "
-            "ON Frames.TimsCalibration=TimsCalibration.Id where "
-            "Frames.Id=%1;")
-      .arg(timsId));
-  if(q.lastError().isValid())
+  auto it_map_record_frames = m_mapFramesRecord.find(timsId);
+  if(it_map_record_frames == m_mapFramesRecord.end())
     {
-
-      throw PappsoException(
-        QObject::tr("ERROR in TIMS sqlite database file %1, database name %2, "
-                    "executing SQL "
-                    "command %3:\n%4\n%5\n%6")
-          .arg(m_timsDataDirectory.absoluteFilePath("analysis.tdf"))
-          .arg(qdb.databaseName())
-          .arg(q.lastQuery())
-          .arg(qdb.lastError().databaseText())
-          .arg(qdb.lastError().driverText())
-          .arg(qdb.lastError().nativeErrorCode()));
+      throw ExceptionNotFound(
+        QObject::tr("ERROR Frames database id %1 not found").arg(timsId));
     }
   pappso::TimsFrameBaseSPtr tims_frame;
-  if(q.next())
+
+
+  tims_frame = std::make_shared<TimsFrameBase>(
+    TimsFrameBase(timsId, it_map_record_frames->second.value(0).toUInt()));
+
+  auto it_map_record = m_mapMzCalibrationRecord.find(
+    it_map_record_frames->second.value(2).toInt());
+  if(it_map_record != m_mapMzCalibrationRecord.end())
     {
 
-
-      tims_frame = std::make_shared<TimsFrameBase>(
-        TimsFrameBase(timsId, q.value(0).toUInt()));
-
-      auto it_map_record = m_mapMzCalibrationRecord.find(q.value(2).toInt());
-      if(it_map_record != m_mapMzCalibrationRecord.end())
-        {
-
-          double T1_frame = q.value(3).toDouble(); // Frames.T1
-          double T2_frame = q.value(4).toDouble(); // Frames.T2
+      double T1_frame =
+        it_map_record_frames->second.value(3).toDouble(); // Frames.T1
+      double T2_frame =
+        it_map_record_frames->second.value(4).toDouble(); // Frames.T2
 
 
-          tims_frame.get()->setMzCalibrationInterfaceSPtr(
-            mpa_mzCalibrationStore->getInstance(
-              T1_frame, T2_frame, it_map_record->second));
-        }
-      else
-        {
-          throw ExceptionNotFound(
-            QObject::tr("ERROR MzCalibration database id %1 not found")
-              .arg(q.value(2).toInt()));
-        }
-
-      tims_frame.get()->setAccumulationTime(q.value(1).toDouble());
-
-      tims_frame.get()->setTime(q.value(5).toDouble());
-      tims_frame.get()->setMsMsType(q.value(6).toUInt());
-
-      tims_frame.get()->setTimsCalibration(q.value(7).toInt(),
-                                           q.value(8).toDouble(),
-                                           q.value(9).toDouble(),
-                                           q.value(10).toDouble(),
-                                           q.value(11).toDouble(),
-                                           q.value(12).toDouble(),
-                                           q.value(13).toDouble(),
-                                           q.value(14).toDouble(),
-                                           q.value(15).toDouble(),
-                                           q.value(16).toDouble(),
-                                           q.value(17).toDouble());
-      return tims_frame;
+      tims_frame.get()->setMzCalibrationInterfaceSPtr(
+        mpa_mzCalibrationStore->getInstance(
+          T1_frame, T2_frame, it_map_record->second));
+    }
+  else
+    {
+      throw ExceptionNotFound(
+        QObject::tr("ERROR MzCalibration database id %1 not found")
+          .arg(it_map_record_frames->second.value(2).toInt()));
     }
 
-  throw ExceptionNotFound(QObject::tr("ERROR timsId %1 not found").arg(timsId));
-  // return TimsFrameCstSPtr;
+  tims_frame.get()->setAccumulationTime(
+    it_map_record_frames->second.value(1).toDouble());
+
+  tims_frame.get()->setTime(it_map_record_frames->second.value(5).toDouble());
+  tims_frame.get()->setMsMsType(it_map_record_frames->second.value(6).toUInt());
+
+
+  auto it_map_record_tims_calibration = m_mapTimsCalibrationRecord.find(
+    it_map_record_frames->second.value(7).toInt());
+  if(it_map_record_tims_calibration != m_mapTimsCalibrationRecord.end())
+    {
+
+      tims_frame.get()->setTimsCalibration(
+        it_map_record_tims_calibration->second.value(1).toInt(),
+        it_map_record_tims_calibration->second.value(2).toDouble(),
+        it_map_record_tims_calibration->second.value(3).toDouble(),
+        it_map_record_tims_calibration->second.value(4).toDouble(),
+        it_map_record_tims_calibration->second.value(5).toDouble(),
+        it_map_record_tims_calibration->second.value(6).toDouble(),
+        it_map_record_tims_calibration->second.value(7).toDouble(),
+        it_map_record_tims_calibration->second.value(8).toDouble(),
+        it_map_record_tims_calibration->second.value(9).toDouble(),
+        it_map_record_tims_calibration->second.value(10).toDouble(),
+        it_map_record_tims_calibration->second.value(11).toDouble());
+    }
+  else
+    {
+      throw ExceptionNotFound(
+        QObject::tr("ERROR TimsCalibration database id %1 not found")
+          .arg(it_map_record_frames->second.value(7).toInt()));
+    }
+
+  return tims_frame;
 }
 
 std::vector<std::size_t>
@@ -413,11 +461,13 @@ TimsData::getTimsMS1FrameIdRange(double rt_begin, double rt_end) const
 {
 
   qDebug() << " rt_begin=" << rt_begin << " rt_end=" << rt_end;
+  if(rt_begin < 0)
+    rt_begin = 0;
   std::vector<std::size_t> tims_frameid_list;
   QSqlDatabase qdb = openDatabaseConnection();
-  QSqlQuery q      = qdb.exec(QString("SELECT Frames.TimsId FROM Frames WHERE "
-                                 "Frames.MsMsType=0 AND (Frames.Time=>%1) AND "
-                                 "(Frames.Time<=%2) SORT BY Frames.Time;")
+  QSqlQuery q      = qdb.exec(QString("SELECT Frames.Id FROM Frames WHERE "
+                                 "Frames.MsMsType=0 AND (Frames.Time>=%1) AND "
+                                 "(Frames.Time<=%2) ORDER BY Frames.Time;")
                            .arg(rt_begin)
                            .arg(rt_end));
   if(q.lastError().isValid())
@@ -446,85 +496,87 @@ TimsFrameCstSPtr
 TimsData::getTimsFrameCstSPtr(std::size_t timsId) const
 {
 
-  qDebug() << " timsId=" << timsId;
+  qDebug() << " timsId=" << timsId
+           << " m_mapFramesRecord.size()=" << m_mapFramesRecord.size();
+  /*
+    for(auto pair_i : m_mapFramesRecord)
+      {
 
-  QSqlDatabase qdb = openDatabaseConnection();
-  QSqlQuery q      = qdb.exec(
-    QString("SELECT Frames.TimsId, Frames.AccumulationTime, " // 1
-            "Frames.MzCalibration, "                          // 2
-            "Frames.T1, Frames.T2, "                          // 4
-            "Frames.Time, Frames.MsMsType, TimsCalibration.ModelType, " // 7
-            "TimsCalibration.C0, TimsCalibration.C1, TimsCalibration.C2, " // 10
-            "TimsCalibration.C3, TimsCalibration.C4, TimsCalibration.C5, " // 13
-            "TimsCalibration.C6, TimsCalibration.C7, TimsCalibration.C8, " // 16
-            "TimsCalibration.C9   FROM  Frames" // 17
-            " INNER JOIN TimsCalibration "
-            "ON Frames.TimsCalibration=TimsCalibration.Id where "
-            "Frames.Id=%1;")
-      .arg(timsId));
-  if(q.lastError().isValid())
+        qDebug() << " pair_i=" << pair_i.first;
+      }
+      */
+
+  auto it_map_record_frames = m_mapFramesRecord.find(timsId);
+  if(it_map_record_frames == m_mapFramesRecord.end())
     {
 
-      throw PappsoException(
-        QObject::tr("ERROR in TIMS sqlite database file %1, database name %2, "
-                    "executing SQL "
-                    "command %3:\n%4\n%5\n%6")
-          .arg(m_timsDataDirectory.absoluteFilePath("analysis.tdf"))
-          .arg(qdb.databaseName())
-          .arg(q.lastQuery())
-          .arg(qdb.lastError().databaseText())
-          .arg(qdb.lastError().driverText())
-          .arg(qdb.lastError().nativeErrorCode()));
+      throw ExceptionNotFound(
+        QObject::tr("ERROR Frames database id %1 not found").arg(timsId));
     }
+
+
   pappso::TimsFrameSPtr tims_frame;
-  if(q.next())
+
+
+  tims_frame = mpa_timsBinDec->getTimsFrameSPtrByOffset(
+    timsId, it_map_record_frames->second.value(0).toUInt());
+
+  qDebug();
+  auto it_map_record = m_mapMzCalibrationRecord.find(
+    it_map_record_frames->second.value(2).toInt());
+  if(it_map_record != m_mapMzCalibrationRecord.end())
     {
 
-
-      tims_frame =
-        mpa_timsBinDec->getTimsFrameSPtrByOffset(timsId, q.value(0).toUInt());
+      double T1_frame =
+        it_map_record_frames->second.value(3).toDouble(); // Frames.T1
+      double T2_frame =
+        it_map_record_frames->second.value(4).toDouble(); // Frames.T2
 
 
-      auto it_map_record = m_mapMzCalibrationRecord.find(q.value(2).toInt());
-      if(it_map_record != m_mapMzCalibrationRecord.end())
-        {
-
-          double T1_frame = q.value(3).toDouble(); // Frames.T1
-          double T2_frame = q.value(4).toDouble(); // Frames.T2
-
-
-          tims_frame.get()->setMzCalibrationInterfaceSPtr(
-            mpa_mzCalibrationStore->getInstance(
-              T1_frame, T2_frame, it_map_record->second));
-        }
-      else
-        {
-          throw ExceptionNotFound(
-            QObject::tr("ERROR MzCalibration database id %1 not found")
-              .arg(q.value(2).toInt()));
-        }
-
-      tims_frame.get()->setAccumulationTime(q.value(1).toDouble());
-
-      tims_frame.get()->setTime(q.value(5).toDouble());
-      tims_frame.get()->setMsMsType(q.value(6).toUInt());
-
-      tims_frame.get()->setTimsCalibration(q.value(7).toInt(),
-                                           q.value(8).toDouble(),
-                                           q.value(9).toDouble(),
-                                           q.value(10).toDouble(),
-                                           q.value(11).toDouble(),
-                                           q.value(12).toDouble(),
-                                           q.value(13).toDouble(),
-                                           q.value(14).toDouble(),
-                                           q.value(15).toDouble(),
-                                           q.value(16).toDouble(),
-                                           q.value(17).toDouble());
-      return tims_frame;
+      tims_frame.get()->setMzCalibrationInterfaceSPtr(
+        mpa_mzCalibrationStore->getInstance(
+          T1_frame, T2_frame, it_map_record->second));
+    }
+  else
+    {
+      throw ExceptionNotFound(
+        QObject::tr("ERROR MzCalibration database id %1 not found")
+          .arg(it_map_record_frames->second.value(2).toInt()));
     }
 
-  throw ExceptionNotFound(QObject::tr("ERROR timsId %1 not found").arg(timsId));
-  // return TimsFrameCstSPtr;
+  tims_frame.get()->setAccumulationTime(
+    it_map_record_frames->second.value(1).toDouble());
+
+  tims_frame.get()->setTime(it_map_record_frames->second.value(5).toDouble());
+  tims_frame.get()->setMsMsType(it_map_record_frames->second.value(6).toUInt());
+
+  qDebug();
+  auto it_map_record_tims_calibration = m_mapTimsCalibrationRecord.find(
+    it_map_record_frames->second.value(7).toInt());
+  if(it_map_record_tims_calibration != m_mapTimsCalibrationRecord.end())
+    {
+
+      tims_frame.get()->setTimsCalibration(
+        it_map_record_tims_calibration->second.value(1).toInt(),
+        it_map_record_tims_calibration->second.value(2).toDouble(),
+        it_map_record_tims_calibration->second.value(3).toDouble(),
+        it_map_record_tims_calibration->second.value(4).toDouble(),
+        it_map_record_tims_calibration->second.value(5).toDouble(),
+        it_map_record_tims_calibration->second.value(6).toDouble(),
+        it_map_record_tims_calibration->second.value(7).toDouble(),
+        it_map_record_tims_calibration->second.value(8).toDouble(),
+        it_map_record_tims_calibration->second.value(9).toDouble(),
+        it_map_record_tims_calibration->second.value(10).toDouble(),
+        it_map_record_tims_calibration->second.value(11).toDouble());
+    }
+  else
+    {
+      throw ExceptionNotFound(
+        QObject::tr("ERROR TimsCalibration database id %1 not found")
+          .arg(it_map_record_frames->second.value(7).toInt()));
+    }
+  qDebug();
+  return tims_frame;
 }
 
 
@@ -561,7 +613,7 @@ TimsData::getPrecursorsFromMzRtCharge(int charge,
   QSqlDatabase qdb = openDatabaseConnection();
   QSqlQuery q      = qdb.exec(
     QString(
-      "SELECT Frames.time, Precursors.MonoisotopicMz, Precursors.Charge, "
+      "SELECT Frames.Time, Precursors.MonoisotopicMz, Precursors.Charge, "
       "Precursors.Id, Frames.Id, PasefFrameMsMsInfo.ScanNumBegin, "
       "PasefFrameMsMsInfo.scanNumEnd "
       "FROM Frames "
@@ -570,7 +622,7 @@ TimsData::getPrecursorsFromMzRtCharge(int charge,
       "WHERE Precursors.Charge == %1 "
       "AND Precursors.MonoisotopicMz > %2 -0.01 "
       "AND Precursors.MonoisotopicMz < %2 +0.01 "
-      "AND Frames.Time >= %3 "
+      "AND Frames.Time >= %3 -1 "
       "AND Frames.Time < %3 +1; ")
       .arg(charge)
       .arg(mz_val)
@@ -591,8 +643,8 @@ TimsData::getPrecursorsFromMzRtCharge(int charge,
     }
   while(q.next())
     {
-      qInfo() << q.value(0).toDouble() << q.value(1).toDouble()
-              << q.value(2).toDouble() << q.value(3).toDouble();
+      // qInfo() << q.value(0).toDouble() << q.value(1).toDouble()
+      //       << q.value(2).toDouble() << q.value(3).toDouble();
 
       std::vector<double> sql_values;
       sql_values.push_back(q.value(4).toDouble()); // frame id
@@ -643,7 +695,7 @@ TimsData::getMatchPrecursorIdByKo(std::vector<std::vector<double>> ids,
       double bko = tims_frame.get()->getOneOverK0Transformation(index[2]);
       double eko = tims_frame.get()->getOneOverK0Transformation(index[3]);
 
-      qInfo() << "diff" << (bko + eko) / 2;
+      // qInfo() << "diff" << (bko + eko) / 2;
       double mean_ko = (bko + eko) / 2;
 
       if(mean_ko > ko_value - 0.1 && mean_ko < ko_value + 0.1)
@@ -958,7 +1010,7 @@ TimsData::getQualifiedMs1MassSpectrumByPrecursorId(std::size_t ms2_index,
             }
         }
     }
-    
+
   catch(PappsoException &error)
     {
       throw error;
@@ -1329,6 +1381,7 @@ TimsData::extractXicListByPrecursorIds(
               xic_structure.scanNumEnd   = q.value(3).toUInt();
               xic_structure.rtTarget     = q.value(0).toDouble();
               // xic_structure.charge       = q.value(5).toUInt();
+              xic_structure.xicSptr = std::make_shared<Xic>();
 
               xic_list.push_back(xic_structure);
             }
@@ -1351,3 +1404,216 @@ TimsData::extractXicListByPrecursorIds(
   qDebug();
   return xic_list;
 }
+
+void
+pappso::TimsData::extractXicListByTimsXicStructureList(
+  std::vector<TimsXicStructure> &tims_xic_structure_list,
+  pappso::XicExtractMethod xicExtractMethod,
+  double rtRange) const
+{
+  try
+    {
+
+      TimsXicExtractorInterface *extractor_p = new TimsDirectXicExtractor(this);
+      extractor_p->setXicExtractMethod(xicExtractMethod);
+      extractor_p->extractTimsXicList(tims_xic_structure_list, rtRange);
+    }
+  catch(PappsoException &error)
+    {
+      throw error;
+    }
+  catch(std::exception &error)
+    {
+      qDebug() << QString("Failure %1 ").arg(error.what());
+    }
+
+  qDebug();
+}
+
+
+pappso::TimsXicStructure
+pappso::TimsData::getTimsXicStructureFromPrecursorId(
+  std::size_t precursor_id, PrecisionPtr precision_ptr) const
+{
+
+  qDebug();
+  TimsXicStructure tims_xic_structure;
+
+  try
+    {
+      QSqlDatabase qdb = openDatabaseConnection();
+      QSqlQuery q      = qdb.exec(
+        QString("SELECT Frames.Time, "
+                "PasefFrameMsMsInfo.Frame,PasefFrameMsMsInfo.ScanNumBegin,"
+                "PasefFrameMsMsInfo.ScanNumEnd, Precursors.MonoisotopicMz, "
+                "Precursors.Charge  FROM "
+                "PasefFrameMsMsInfo INNER JOIN Precursors ON "
+                "PasefFrameMsMsInfo.Precursor=Precursors.Id INNER JOIN "
+                "Frames ON PasefFrameMsMsInfo.Frame=Frames.Id where "
+                "Precursors.Id=%1;")
+          .arg(precursor_id));
+      if(q.lastError().isValid())
+        {
+          qDebug();
+          throw PappsoException(
+            QObject::tr("ERROR in TIMS sqlite database file %1, executing SQL "
+                        "command %2:\n%3\n%4\n%5")
+              .arg(m_timsDataDirectory.absoluteFilePath("analysis.tdf"))
+              .arg(q.lastQuery())
+              .arg(qdb.lastError().databaseText())
+              .arg(qdb.lastError().driverText())
+              .arg(qdb.lastError().nativeErrorCode()));
+        }
+
+      while(q.next())
+        {
+          // Frame, ScanNumBegin, ScanNumEnd, IsolationMz, ColliqionEnergy,
+          // Precursor
+          // Id, LargestPeakMz, AverageMz, MonoisotopicMz, Charge,
+          // ScanNumber, Intensity, Parent
+
+          tims_xic_structure.precursorId = precursor_id;
+          tims_xic_structure.mzRange =
+            MzRange(q.value(4).toDouble(), precision_ptr);
+          tims_xic_structure.scanNumBegin = q.value(2).toUInt();
+          tims_xic_structure.scanNumEnd   = q.value(3).toUInt();
+          tims_xic_structure.rtTarget     = q.value(0).toDouble();
+          // xic_structure.charge       = q.value(5).toUInt();
+          tims_xic_structure.xicSptr = std::make_shared<Xic>();
+        }
+    }
+  catch(PappsoException &error)
+    {
+      throw error;
+    }
+  catch(std::exception &error)
+    {
+      qDebug() << QString("Failure %1 ").arg(error.what());
+    }
+  return tims_xic_structure;
+}
+
+
+std::map<quint32, quint32>
+TimsData::getRawMs2ByPrecursorId(std::size_t precursor_index)
+{
+  qDebug();
+  std::map<quint32, quint32> raw_spectrum;
+  try
+    {
+      QSqlDatabase qdb = openDatabaseConnection();
+
+      qdb = openDatabaseConnection();
+      QSqlQuery q =
+        qdb.exec(QString("SELECT PasefFrameMsMsInfo.*, Precursors.* FROM "
+                         "PasefFrameMsMsInfo INNER JOIN Precursors ON "
+                         "PasefFrameMsMsInfo.Precursor=Precursors.Id where "
+                         "Precursors.Id=%1;")
+                   .arg(precursor_index));
+      if(q.lastError().isValid())
+        {
+          qDebug();
+          throw PappsoException(
+            QObject::tr("ERROR in TIMS sqlite database file %1, executing SQL "
+                        "command %2:\n%3\n%4\n%5")
+              .arg(m_timsDataDirectory.absoluteFilePath("analysis.tdf"))
+              .arg(q.lastQuery())
+              .arg(qdb.lastError().databaseText())
+              .arg(qdb.lastError().driverText())
+              .arg(qdb.lastError().nativeErrorCode()));
+        }
+      qDebug();
+      // m_mutex.unlock();
+      if(q.size() == 0)
+        {
+
+          throw ExceptionNotFound(
+            QObject::tr(
+              "ERROR in getQualifiedMassSpectrumByPrecursorId, precursor "
+              "id=%1 not found")
+              .arg(precursor_index));
+        }
+      else
+        {
+          //  qDebug() << " q.size()="<< q.size();
+          qDebug();
+          bool first                      = true;
+          std::size_t scan_mobility_start = 0;
+          std::size_t scan_mobility_end   = 0;
+          std::vector<std::size_t> tims_frame_list;
+
+          while(q.next())
+            {
+              tims_frame_list.push_back(q.value(0).toLongLong());
+              if(first)
+                {
+
+                  scan_mobility_start = q.value(1).toLongLong();
+                  scan_mobility_end   = q.value(2).toLongLong();
+
+                  first = false;
+                }
+            }
+          // QMutexLocker locker(&m_mutex_spectrum);
+          qDebug();
+          pappso::TimsFrameCstSPtr tims_frame, previous_frame;
+          // TracePlusCombiner combiner;
+          // MapTrace combiner_result;
+          for(std::size_t tims_id : tims_frame_list)
+            {
+              tims_frame = getTimsFrameCstSPtrCached(tims_id);
+              qDebug();
+              /*combiner.combine(combiner_result,
+                               tims_frame.get()->cumulateScanToTrace(
+                                 scan_mobility_start, scan_mobility_end));*/
+              if(previous_frame.get() != nullptr)
+                {
+                  if(previous_frame.get()->hasSameCalibrationData(
+                       *tims_frame.get()))
+                    {
+                    }
+                  else
+                    {
+                      throw ExceptionNotFound(
+                        QObject::tr(
+                          "ERROR in %1 %2, different calibration data "
+                          "between frame id %3 and frame id %4")
+                          .arg(__FILE__)
+                          .arg(__FUNCTION__)
+                          .arg(previous_frame.get()->getId())
+                          .arg(tims_frame.get()->getId()));
+                    }
+                }
+              tims_frame.get()->cumulateScansInRawMap(
+                raw_spectrum, scan_mobility_start, scan_mobility_end);
+              qDebug();
+
+              previous_frame = tims_frame;
+            }
+          qDebug() << " precursor_index=" << precursor_index
+                   << " num_rows=" << tims_frame_list.size()
+                   << " sql=" << q.lastQuery() << " "
+                   << (std::size_t)QThread::currentThreadId();
+          if(first == true)
+            {
+              throw ExceptionNotFound(
+                QObject::tr(
+                  "ERROR in getQualifiedMassSpectrumByPrecursorId, precursor "
+                  "id=%1 not found")
+                  .arg(precursor_index));
+            }
+          qDebug();
+        }
+    }
+
+  catch(PappsoException &error)
+    {
+      throw error;
+    }
+  catch(std::exception &error)
+    {
+      qDebug() << QString("Failure %1 ").arg(error.what());
+    }
+  return raw_spectrum;
+  qDebug();
+}
