@@ -49,12 +49,32 @@ struct TimsXicStructure
 {
   TimsXicStructure()
     : mzRange(pappso_double(1), PrecisionFactory::getPpmInstance(10.0)){};
+  /** @brief the corresponding precursor id
+   */
   std::size_t precursorId;
+
+  /** @brief mass range to extract
+   */
   MzRange mzRange;
+  /** @brief mobility index begin
+   */
   std::size_t scanNumBegin;
+
+  /** @brief mobility index end
+   */
   std::size_t scanNumEnd;
+
+  /** @brief targeted retention time for this precursor
+   */
   pappso::pappso_double rtTarget;
+
+  /** @brief extracted xic
+   */
   XicSPtr xicSptr = nullptr;
+
+  /** @brief targeted charge for this precursor
+   */
+
   unsigned int charge;
 };
 
@@ -161,7 +181,51 @@ class PMSPP_LIB_DECL TimsData
    */
   TimsFrameCstSPtr getTimsFrameCstSPtr(std::size_t timsId) const;
 
+  TimsXicStructure
+  getTimsXicStructureFromPrecursorId(std::size_t precursor_id,
+                                     PrecisionPtr precision_ptr) const;
+
+
+  /** @brief extract a list of XICs from Tims data
+   *
+   * @param tims_xic_structure_list the list of tims XIC structure  to extract
+   * @param xicExtractMethod XIC extraction method (sum or max) to use
+   * @param rtRange retention time range in seconds to extract XIC from rtTarget
+   * - rtRange to rtTarget + rtRange
+   */
+  void extractXicListByTimsXicStructureList(
+    std::vector<TimsXicStructure> &tims_xic_structure_list,
+    XicExtractMethod xicExtractMethod,
+    double rtRange) const;
+
+
+  /** @brief get cumulated raw signal for a given precursor
+   * only to use to see the raw signal
+   *
+   * @param precursor_index precursor index to extract signal from
+   * @result a map of integers, x=time of flights, y= intensities
+   */
+  std::map<quint32, quint32>
+  getRawMs2ByPrecursorId(std::size_t precursor_index);
+
   private:
+  /** @brief extract a list of XICs from Tims data
+   *
+   * @param precursor_id_list the list of precursors to extract
+   * @param precision_ptr precision to compute the mz range to extract for each
+   * precursor mass
+   * @param xicExtractMethod XIC extraction method (sum or max) to use
+   * @param rtRange retention time range in seconds to extract XIC from rtTarget
+   * - rtRange to rtTarget + rtRange
+   * @result the corresponding XIC list as a specific structure
+   */
+  std::vector<TimsXicStructure> extractXicListByPrecursorIds(
+    const std::vector<std::size_t> &precursor_id_list,
+    PrecisionPtr precision_ptr,
+    XicExtractMethod xicExtractMethod,
+    double rtRange) const;
+
+
   std::pair<std::size_t, std::size_t>
   getScanCoordinateFromRawIndex(std::size_t spectrum_index) const;
 
@@ -184,21 +248,6 @@ class PMSPP_LIB_DECL TimsData
 
   TimsFrameBaseCstSPtr getTimsFrameBaseCstSPtrCached(std::size_t timsId);
 
-  /** @brief extract a list of XICs from Tims data
-   *
-   * @param precursor_id_list the list of precursors to extract
-   * @param precision_ptr precision to compute the mz range to extract for each
-   * precursor mass
-   * @param xicExtractMethod XIC extraction method (sum or max) to use
-   * @param rtRange retention time range in seconds to extract XIC from rtTarget
-   * - rtRange to rtTarget + rtRange
-   * @result the corresponding XIC list as a specific structure
-   */
-  std::vector<TimsXicStructure> extractXicListByPrecursorIds(
-    const std::vector<std::size_t> &precursor_id_list,
-    PrecisionPtr precision_ptr,
-    XicExtractMethod xicExtractMethod,
-    double rtRange) const;
 
   std::vector<std::size_t>
   getMatchPrecursorIdByKo(std::vector<std::vector<double>> ids,
@@ -210,6 +259,12 @@ class PMSPP_LIB_DECL TimsData
   getClosestPrecursorIdByMz(std::vector<std::vector<double>> ids,
                             double mz_value);
 
+
+  /** @brief private function to fill m_frameIdDescrList
+   */
+  void fillFrameIdDescrList();
+
+
   private:
   QDir m_timsDataDirectory;
   TimsBinDec *mpa_timsBinDec = nullptr;
@@ -229,8 +284,27 @@ class PMSPP_LIB_DECL TimsData
 
 
   std::map<int, QSqlRecord> m_mapMzCalibrationRecord;
+  std::map<int, QSqlRecord> m_mapTimsCalibrationRecord;
+  std::map<std::size_t, QSqlRecord> m_mapFramesRecord;
 
-  MzCalibrationStore * mpa_mzCalibrationStore;
+  MzCalibrationStore *mpa_mzCalibrationStore;
+
+
+  struct FrameIdDescr
+  {
+    std::size_t m_frameId;   // frame id
+    std::size_t m_size;      // frame size
+    std::size_t m_cumulSize; // cumulative size
+  };
+
+  /** @brief store every frame id and corresponding sizes
+   */
+  std::vector<FrameIdDescr> m_frameIdDescrList;
+
+  /** @brief index to find quickly a frameId in the description list with the
+   * raw index of spectrum modulo 1000
+   */
+  std::map<std::size_t, std::size_t> m_thousandIndexToFrameIdDescrListIndex;
 
   QMutex m_mutex;
 };
