@@ -406,11 +406,22 @@ TimsFrame::extractTimsXicListInRtRange(
   std::vector<TimsXicStructure>::iterator &itXicListend,
   XicExtractMethod method) const
 {
+  qDebug() << std::distance(itXicListbegin, itXicListend);
   std::vector<TimsFrame::XicComputeStructure> tmp_xic_list;
 
   for(auto it = itXicListbegin; it != itXicListend; it++)
     {
       tmp_xic_list.push_back(TimsFrame::XicComputeStructure(this, *it));
+
+      qDebug() << " tmp_xic_struct.mobilityIndexBegin="
+               << tmp_xic_list.back().mobilityIndexBegin
+               << " tmp_xic_struct.mobilityIndexEnd="
+               << tmp_xic_list.back().mobilityIndexEnd;
+
+      qDebug() << " tmp_xic_struct.mzIndexLowerBound="
+               << tmp_xic_list.back().mzIndexLowerBound
+               << " tmp_xic_struct.mzIndexUpperBound="
+               << tmp_xic_list.back().mzIndexUpperBound;
     }
   if(tmp_xic_list.size() == 0)
     return;
@@ -438,6 +449,7 @@ TimsFrame::extractTimsXicListInRtRange(
   while(it_scan_num != it_scan_num_end)
     {
       TraceSPtr ms_spectrum = getRawTraceSPtr(*it_scan_num);
+      // qDebug() << ms_spectrum.get()->toString();
       for(auto &&tmp_xic_struct : tmp_xic_list)
         {
           if(((*it_scan_num) >= tmp_xic_struct.mobilityIndexBegin) &&
@@ -448,6 +460,9 @@ TimsFrame::extractTimsXicListInRtRange(
                   tmp_xic_struct.tmpIntensity +=
                     ms_spectrum.get()->maxY(tmp_xic_struct.mzIndexLowerBound,
                                             tmp_xic_struct.mzIndexUpperBound);
+
+                  qDebug() << "tmp_xic_struct.tmpIntensity="
+                           << tmp_xic_struct.tmpIntensity;
                 }
               else
                 {
@@ -455,6 +470,8 @@ TimsFrame::extractTimsXicListInRtRange(
                   tmp_xic_struct.tmpIntensity +=
                     ms_spectrum.get()->sumY(tmp_xic_struct.mzIndexLowerBound,
                                             tmp_xic_struct.mzIndexUpperBound);
+                  qDebug() << "tmp_xic_struct.tmpIntensity="
+                           << tmp_xic_struct.tmpIntensity;
                 }
             }
         }
@@ -465,10 +482,13 @@ TimsFrame::extractTimsXicListInRtRange(
     {
       if(tmp_xic_struct.tmpIntensity != 0)
         {
+          qDebug() << tmp_xic_struct.xic_ptr;
           tmp_xic_struct.xic_ptr->push_back(
             {m_time, tmp_xic_struct.tmpIntensity});
         }
     }
+
+  qDebug();
 }
 
 
@@ -476,14 +496,14 @@ pappso::TraceSPtr
 TimsFrame::getRawTraceSPtr(std::size_t scanNum) const
 {
 
-  qDebug();
+  // qDebug();
 
   pappso::TraceSPtr trace_sptr = std::make_shared<pappso::Trace>();
   // std::vector<DataPoint>
 
   if(m_timsDataFrame.size() == 0)
     return trace_sptr;
-  qDebug();
+  // qDebug();
 
   std::size_t size = getNbrPeaks(scanNum);
 
@@ -503,8 +523,9 @@ TimsFrame::getRawTraceSPtr(std::size_t scanNum) const
       data_point.y *= 100.0 / m_accumulationTime;
 
       previous = data_point.x;
+      trace_sptr.get()->push_back(data_point);
     }
-  qDebug();
+  // qDebug();
   return trace_sptr;
 }
 
