@@ -26,6 +26,7 @@
  ******************************************************************************/
 
 #include "timsdirectxicextractor.h"
+#include <QDebug>
 
 using namespace pappso;
 
@@ -43,6 +44,7 @@ void
 TimsDirectXicExtractor::extractTimsXicList(
   std::vector<TimsXicStructure> &timsXicList, double rtRange) const
 {
+  qDebug();
   if(timsXicList.size() == 0)
     return;
   std::sort(timsXicList.begin(),
@@ -51,28 +53,59 @@ TimsDirectXicExtractor::extractTimsXicList(
               return a.rtTarget < b.rtTarget;
             });
 
+  for(auto &&timsXic : timsXicList)
+    {
+      timsXic.xicSptr = std::make_shared<Xic>();
+    }
+
   std::vector<std::size_t> tims_frameid_list =
     mp_timsData->getTimsMS1FrameIdRange(timsXicList[0].rtTarget - rtRange,
                                         timsXicList.back().rtTarget + rtRange);
 
-  std::vector<TimsXicStructure>::iterator itXicListbegin = timsXicList.begin();
-  std::vector<TimsXicStructure>::iterator itXicListend   = timsXicList.begin();
-
+  qDebug() << " tims_frameid_list.size()=" << tims_frameid_list.size();
+  qDebug() << " rt begin=" << timsXicList[0].rtTarget;
+  qDebug() << " rt end=" << timsXicList.back().rtTarget;
   for(std::size_t frame_id : tims_frameid_list)
     {
+      std::vector<TimsXicStructure>::iterator itXicListbegin =
+        timsXicList.begin();
+      std::vector<TimsXicStructure>::iterator itXicListend = timsXicList.end();
       TimsFrameCstSPtr frame_sptr = mp_timsData->getTimsFrameCstSPtr(frame_id);
-      double rt                   = frame_sptr.get()->getTime();
-      while((itXicListbegin != timsXicList.end()) &&
-            ((itXicListbegin->rtTarget - rtRange) < rt))
+      double rtframe              = frame_sptr.get()->getTime();
+
+      double rtbeginframe = rtframe - rtRange;
+      double rtendframe   = rtframe + rtRange;
+
+      if(rtbeginframe < 0)
+        rtbeginframe = 0;
+      while((itXicListbegin != itXicListend) &&
+            (itXicListbegin->rtTarget < rtbeginframe))
         {
           itXicListbegin++;
         }
+      itXicListend = itXicListbegin;
       while((itXicListend != timsXicList.end()) &&
-            (rt < (itXicListend->rtTarget + rtRange)))
+            (itXicListend->rtTarget < rtendframe))
         {
           itXicListend++;
         }
+      /*
+      while((itXicListbegin != timsXicList.end()) &&
+            ((itXicListbegin->rtTarget + rtRange) < rtframe))
+        {
+
+          qDebug() << " itXicListbegin->rtTarget + rtRange="
+                   << itXicListbegin->rtTarget + rtRange
+                   << " rtframe=" << rtframe;
+          itXicListbegin++;
+        }
+*/
+      qDebug() << " from=" << itXicListbegin->rtTarget
+               << " to=" << itXicListend->rtTarget;
       frame_sptr.get()->extractTimsXicListInRtRange(
         itXicListbegin, itXicListend, m_xicExtractMethod);
+      
+      qDebug() << "" << frame_sptr.get()->getId();
     }
+  qDebug();
 }
