@@ -318,7 +318,6 @@ MassSpectrumWidget::plot()
     }
   else
     {
-
       _peak_ion_isotope_match_list.sort(
         [](const PeakIonIsotopeMatch &a, const PeakIonIsotopeMatch &b) {
           return a.getPeak().y > b.getPeak().y;
@@ -455,3 +454,18 @@ MassSpectrumWidget::computeIsotopeMassList()
         }
     }
 }
+
+void
+pappso::MassSpectrumWidget::highlightPrecursorPeaks()
+{
+
+  if(_ms_level > 1)
+    {
+      pappso_double precursor_mz_1 = _peptide_sp->getMz(1);
+      _custom_plot->highlightPrecursorPeaks(
+        precursor_mz_1, 1, _p_ms2_precision);
+      pappso_double precursor_mz_charge = _peptide_sp->getMz(_peptide_charge);
+      _custom_plot->highlightPrecursorPeaks(
+        precursor_mz_charge, _peptide_charge, _p_ms2_precision);
+    }
+}
