@@ -37,8 +37,13 @@ struct PMSPP_LIB_DECL ColorMapPlotConfig
   double minMzValue = std::numeric_limits<double>::max();
   double maxMzValue = std::numeric_limits<double>::min();
 
+  double minZValue = std::numeric_limits<double>::max();
+  double maxZValue = std::numeric_limits<double>::min();
+
   ColorMapPlotConfig();
 
+  ColorMapPlotConfig(const ColorMapPlotConfig &other);
+
   ColorMapPlotConfig(DataKind x_axis_data_kind,
                      DataKind y_axis_data_kind,
 
@@ -53,8 +58,12 @@ struct PMSPP_LIB_DECL ColorMapPlotConfig
                      double max_key_value,
 
                      double min_mz_value,
-                     double max_mz_value);
+                     double max_mz_value,
 
+                     double min_z_value,
+                     double max_z_value);
+
+  ColorMapPlotConfig &operator=(const ColorMapPlotConfig &other);
   QString toString() const;
 };
 
